;+
;   
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU Lesser General Public License as published
;    by the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU Lesser General Public License for more details.
;
;    You should have received a copy of the GNU Lesser General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;-

;+
; v.20131112a
;-

pro hw_alpha_doit, ev

  compile_opt idl2

  catch, theerror
  if theerror ne 0 then begin
    catch, /cancel
    void = dialog_message(!error_state.msg,/error,/center)
    widget_control, info.run_id, sensitive=0
    close, /all
    return
  endif

  ; update program status
  widget_control, ev.top, get_uvalue = info, /no_copy

  ; retrieve the input OMEGA file, softXX directory,
  ; output directory and output options
  widget_control, info.omgpath_id, get_value=omgpath
  widget_control, info.qubpath_id, get_value=qubpath
  widget_control, info.navpath_id, get_value=navpath
  widget_control, info.savpath_id, get_value=savpath
  widget_control, info.options_id, get_value=options
 
  ; disable run button as long as the module runs
  widget_control, info.run_id, sensitive=0

  nfiles = n_elements(qubpath)
  print, nfiles

  ;; ADJUST OMEGA_PATH in SOFTXX

  ; write the OMEGA QUB and NAV paths to file 'omega_path' in the SOFTXX directory
  openw,  lun, omgpath+path_sep()+'omega_path', /get_lun
 
  printf, lun, file_dirname(qubpath)+path_sep()
  printf, lun, file_dirname(navpath)+path_sep()
 
  free_lun, lun


  ;; CALL READOMEGA IN SOFTXX

  ; call SOFTXX (via the newly added header patch)
  cd, current = orgpath
  cd, omgpath
 
  ; strip filename from extension
  basename = (strmid(file_basename(qubpath),0,9))[0]
  hw_alpha_omega,basename,idat,jdat,geocube,alt,lat,lon,specmars,orbnum,wl,ic
 
  ; create a bad band list based on SOFTXX variable 'ic'
  bbl = bytarr(352)
  bbl[ic] = 1
 
  cd, orgpath

  ;; OUTPUT OMEGA DATA TO ENVI FILES

  ; write raw data
  if options[0] eq 1 then begin
    fname = savpath+basename+'_idat'
    desc = 'OMEGA raw DN data'
    bnames = 'Band '+strtrim(indgen(352),2)
    hw_alpha_save_binary, fname, idat
    hw_alpha_save_header, fname, desc, (size(idat))[1], $
      (size(idat))[3], 352, size(idat,/type), $
      bnames=bnames,bbl=bbl, wl=wl
  endif

  ; write radiance data
  if options[1] eq 1 then begin
    fname = savpath+basename+'_jdat'
    desc = 'OMEGA radiance data'
    bnames = 'Band '+strtrim(indgen(352),2)
    hw_alpha_save_binary, fname, jdat
    hw_alpha_save_header, fname, desc, (size(jdat))[1], $
      (size(jdat))[3], 352, size(jdat,/type), $
      bnames=bnames,bbl=bbl, wl=wl
  endif
 
  ; write MOLA altitude data
  if options[2] eq 1 then begin
    fname = savpath+basename+'_mola'
    desc = 'MOLA altitude data'
    bnames = 'Altitude'
    hw_alpha_save_binary, fname, alt
    hw_alpha_save_header, fname, desc, (size(alt))[1], $
      (size(alt))[2], 1, size(alt,/type), bnames = bnames
  endif
   
  ; write latlon data
  if options[3] eq 1 then begin
    fname = savpath+basename+'_lonlat'
    desc = 'OMEGA longitude and latitude'
    bnames = ['Longitude','Latitude']
    hw_alpha_save_binary, fname, transpose([ [[lon]],[[lat]] ],[0,2,1])
    hw_alpha_save_header, fname, desc, (size(lat))[1], $
     (size(lat))[2], 2, size(lon,/type), bnames=bnames
  endif

  ; write geocube data
  if options[4] eq 1 then begin
    fname = savpath+basename+'_geocube'
    desc = 'OMEGA GeoCUBE'
    bnames = [ $
     'scan value (in DN) for each pixel',$                                  ;00
     'time (hr:min:sec:msec)',$                          ;01
     'incidence on ellipsoid  (unit: 0.0001°)',$                        ;02
     'emergence from ellipsoid (unit: 0.0001°)',$                       ;03
     'incidence wrt local gravity field (atmospheric science)',$        ;04
     'emergence wrt local gravity field',$                              ;05
     ; IR C pixel
     'longitude IR C pixel center (0.0001°)',$                ;06
     'latitude IR C pixel center (0.0001°)',$                 ;07
     'incidence for the IR C pixel (unit: 0.0001°)',$                       ;08
     'emergence for the IR C pixel (unit: 0.0001°)',$                       ;09
     'phase for the IR C pixel (unit: 0.0001°)',$                           ;10
     'distance in m', $                                                     ;11
     'altitude (MOLA - ellipsoid) in m',$                                   ;12
       ;if > 65.535 km: altitude above the surface + 65536 m (for limb observations)
     replicate('longitudes of the corners of the IFOV (unit: 0.0001°)',4),$ ;13 - 16
     replicate('latitudes of the corners of the IFOV (unit: 0.0001°)' ,4),$ ;17 - 20 
     ; IR L pixel (21-35: same as 6-20 for the IR L pixel)
     'longitude of the center of the IR L pixel (0.0001°)',$                ;21
     'latitude of the center of the IR L pixel (0.0001°)',$                 ;22
     'incidence for the IR L pixel (unit: 0.0001°)',$                       ;23
     'emergence for the IR L pixel (unit: 0.0001°)',$                       ;24
     'phase for the IR L pixel (unit: 0.0001°)',$                           ;25
     'distance in m', $                                                     ;26
     'altitude (MOLA - ellipsoid) in m',$                                   ;27
     replicate('longitudes of the corners of the IFOV (unit: 0.0001°)',4),$ ;28 - 31
     replicate('latitudes of the corners of the IFOV (unit: 0.0001°)' ,4),$ ;32 - 35 
     ; Vis pixel (36-50: same as 6-20 for the Visible pixel)
     'longitude of the center of the VIS pixel (0.0001°)',$                 ;36
     'latitude of the center of the VIS pixel (0.0001°)',$                  ;37
     'incidence for the VIS pixel (unit: 0.0001°)',$                        ;38
     'emergence for the VIS pixel (unit: 0.0001°)',$                        ;39
     'phase for the VIS pixel (unit: 0.0001°)',$                            ;40
     'distance in m', $                                                     ;41
     'altitude (MOLA - ellipsoid) in m',$                                   ;42
     replicate('longitudes of the corners of the IFOV (unit: 0.0001°)',4),$ ;43 - 46
     replicate('latitudes of the corners of the IFOV (unit: 0.0001°)' ,4)]  ;47 - 50 
    hw_alpha_save_binary, fname, geocube
    hw_alpha_save_header, fname, desc, (size(geocube))[1], $
      (size(geocube))[3], 51, size(geocube,/type), bnames = bnames
  endif
   
  ; write specmars data to ASCII
  if options[5] eq 1 then begin
    fname = savpath+basename+'_specmars.txt'
    openw,  lun, fname, /get_lun
    for i = 0, 351 do printf,lun,strtrim(wl[i],2)," ",strtrim(specmars[i],2)
    free_lun, lun
  endif

  ; clear selected file after running, disable run button
  widget_control, info.run_id, sensitive = 1
  widget_control, ev.top, set_uvalue = info

end
